@@ -2,21 +2,7 @@
  *
  * Copyright (C) 2015 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 /**
@@ -33,7 +19,6 @@
 
 #include "config.h"
 
-#include <fnmatch.h>
 #include <string.h>
 
 #include "as-app-builder.h"
@@ -159,7 +144,7 @@ as_app_builder_search_locale_gettext (AsAppBuilderContext *ctx,
 	if (dir == NULL)
 		return FALSE;
 
-	/* do a first pass at this, trying to find the prefered .mo */
+	/* do a first pass at this, trying to find the preferred .mo */
 	mo_paths = g_ptr_array_new_with_free_func (g_free);
 	while ((filename = g_dir_read_name (dir)) != NULL) {
 		g_autofree gchar *path = NULL;
