@@ -2,24 +2,9 @@
  *
  * Copyright (C) 2013-2015 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
-#include <fnmatch.h>
 #include <string.h>
 
 #include "as-app-private.h"
@@ -44,18 +29,20 @@ as_app_desktop_key_get_locale (const gchar *key)
 }
 
 static gboolean
-as_app_infer_file_key (AsApp *app,
-		       GKeyFile *kf,
-		       const gchar *key,
-		       GError **error)
+as_app_infer_kudos (AsApp *app, GKeyFile *kf, const gchar *key, GError **error)
 {
-	g_autofree gchar *tmp = NULL;
-
 	if (g_strcmp0 (key, "X-GNOME-UsesNotifications") == 0) {
 		as_app_add_kudo_kind (AS_APP (app),
 				      AS_KUDO_KIND_NOTIFICATIONS);
+	}
+	return TRUE;
+}
 
-	} else if (g_strcmp0 (key, "X-GNOME-Bugzilla-Bugzilla") == 0) {
+static gboolean
+as_app_infer_project_group (AsApp *app, GKeyFile *kf, const gchar *key, GError **error)
+{
+	g_autofree gchar *tmp = NULL;
+	if (g_strcmp0 (key, "X-GNOME-Bugzilla-Bugzilla") == 0) {
 		tmp = g_key_file_get_string (kf,
 					     G_KEY_FILE_DESKTOP_GROUP,
 					     key,
@@ -114,11 +101,11 @@ as_app_parse_file_metadata (AsApp *app, GKeyFile *kf, const gchar *key)
 	guint i;
 	g_autofree gchar *value = NULL;
 	const gchar *blacklist[] = {
-		"X-AppInstall-*",
+		"X-AppInstall-",
 		"X-Desktop-File-Install-Version",
-		"X-Geoclue-Reason*",
-		"X-GNOME-Bugzilla-*",
-		"X-GNOME-FullName*",
+		"X-Geoclue-Reason",
+		"X-GNOME-Bugzilla-",
+		"X-GNOME-FullName",
 		"X-GNOME-Gettext-Domain",
 		"X-GNOME-UsesNotifications",
 		NULL };
@@ -128,7 +115,7 @@ as_app_parse_file_metadata (AsApp *app, GKeyFile *kf, const gchar *key)
 
 	/* anything blacklisted */
 	for (i = 0; blacklist[i] != NULL; i++) {
-		if (fnmatch (blacklist[i], key, 0) == 0)
+		if (g_str_has_prefix (blacklist[i], key))
 			return;
 	}
 	value = g_key_file_get_string (kf,
@@ -468,6 +455,78 @@ as_app_parse_file_key_fallback_comment (AsApp *app,
 	return TRUE;
 }
 
+gboolean
+as_app_parse_desktop_kf (AsApp *app, GKeyFile *kf, AsAppParseFlags flags, GError **error)
+{
+	g_auto(GStrv) keys = NULL;
+
+	/* check this is a valid desktop file */
+	if (!g_key_file_has_group (kf, G_KEY_FILE_DESKTOP_GROUP)) {
+		g_set_error (error,
+			     AS_APP_ERROR,
+			     AS_APP_ERROR_INVALID_TYPE,
+			     "Not a desktop file: no [%s]",
+			     G_KEY_FILE_DESKTOP_GROUP);
+		return FALSE;
+	}
+
+	/* look at all the keys */
+	keys = g_key_file_get_keys (kf, G_KEY_FILE_DESKTOP_GROUP, NULL, error);
+	if (keys == NULL)
+		return FALSE;
+	as_app_set_kind (app, AS_APP_KIND_DESKTOP);
+	for (guint i = 0; keys[i] != NULL; i++) {
+		if (!as_app_parse_file_key (app, kf, keys[i], flags, error))
+			return FALSE;
+		if ((flags & AS_APP_PARSE_FLAG_USE_HEURISTICS) > 0) {
+			if (!as_app_infer_kudos (app, kf, keys[i], error))
+				return FALSE;
+			if (as_app_get_project_group (app) == NULL) {
+				if (!as_app_infer_project_group (app, kf, keys[i], error))
+					return FALSE;
+			}
+		}
+	}
+
+	/* perform any fallbacks */
+	if ((flags & AS_APP_PARSE_FLAG_USE_FALLBACKS) > 0 &&
+	    as_app_get_comment_size (app) == 0) {
+		for (guint i = 0; keys[i] != NULL; i++) {
+			if (!as_app_parse_file_key_fallback_comment (app,
+								     kf,
+								     keys[i],
+								     error))
+				return FALSE;
+		}
+	}
+
+	/* all applications require icons */
+	if (as_app_get_icons(app)->len == 0)
+		as_app_add_veto (app, "has no icon");
+	return TRUE;
+}
+
+gboolean
+as_app_parse_desktop_data (AsApp *app, GBytes *data, AsAppParseFlags flags, GError **error)
+{
+	GKeyFileFlags kf_flags = G_KEY_FILE_KEEP_TRANSLATIONS;
+	g_autoptr(GError) error_local = NULL;
+	g_autoptr(GKeyFile) kf = NULL;
+
+	kf = g_key_file_new ();
+	if (flags & AS_APP_PARSE_FLAG_KEEP_COMMENTS)
+		kf_flags |= G_KEY_FILE_KEEP_COMMENTS;
+	if (!g_key_file_load_from_bytes (kf, data, kf_flags, &error_local)) {
+		g_set_error (error,
+			     AS_APP_ERROR,
+			     AS_APP_ERROR_INVALID_TYPE,
+			     "Failed to parse data: %s",
+			     error_local->message);
+		return FALSE;
+	}
+	return as_app_parse_desktop_kf (app, kf, flags, error);
+}
+
 gboolean
 as_app_parse_desktop_file (AsApp *app,
 			   const gchar *desktop_file,
@@ -476,11 +535,9 @@ as_app_parse_desktop_file (AsApp *app,
 {
 	GKeyFileFlags kf_flags = G_KEY_FILE_KEEP_TRANSLATIONS;
 	gchar *tmp;
-	guint i;
-	g_autoptr(GError) error_local = NULL;
 	g_autofree gchar *app_id = NULL;
+	g_autoptr(GError) error_local = NULL;
 	g_autoptr(GKeyFile) kf = NULL;
-	g_auto(GStrv) keys = NULL;
 
 	/* load file */
 	kf = g_key_file_new ();
@@ -495,20 +552,6 @@ as_app_parse_desktop_file (AsApp *app,
 		return FALSE;
 	}
 
-	/* check this is a valid desktop file */
-	if (!g_key_file_has_group (kf, G_KEY_FILE_DESKTOP_GROUP)) {
-		g_set_error (error,
-			     AS_APP_ERROR,
-			     AS_APP_ERROR_INVALID_TYPE,
-			     "Not a desktop file: no [%s]",
-			     G_KEY_FILE_DESKTOP_GROUP);
-		return FALSE;
-	}
-
-	/* create app */
-	app_id = g_path_get_basename (desktop_file);
-	as_app_set_kind (app, AS_APP_KIND_DESKTOP);
-
 	/* is this really a web-app? */
 	if ((flags & AS_APP_PARSE_FLAG_USE_HEURISTICS) > 0) {
 		g_autofree gchar *exec = NULL;
@@ -523,40 +566,11 @@ as_app_parse_desktop_file (AsApp *app,
 	}
 
 	/* Ubuntu helpfully put the package name in the desktop file name */
+	app_id = g_path_get_basename (desktop_file);
 	tmp = g_strstr_len (app_id, -1, ":");
 	if (tmp != NULL)
 		as_app_set_id (app, tmp + 1);
 	else
 		as_app_set_id (app, app_id);
-
-	/* look at all the keys */
-	keys = g_key_file_get_keys (kf, G_KEY_FILE_DESKTOP_GROUP, NULL, error);
-	if (keys == NULL)
-		return FALSE;
-	for (i = 0; keys[i] != NULL; i++) {
-		if (!as_app_parse_file_key (app, kf, keys[i], flags, error))
-			return FALSE;
-		if ((flags & AS_APP_PARSE_FLAG_USE_HEURISTICS) > 0) {
-			if (!as_app_infer_file_key (app, kf, keys[i], error))
-				return FALSE;
-		}
-	}
-
-	/* perform any fallbacks */
-	if ((flags & AS_APP_PARSE_FLAG_USE_FALLBACKS) > 0 &&
-	    as_app_get_comment_size (app) == 0) {
-		for (i = 0; keys[i] != NULL; i++) {
-			if (!as_app_parse_file_key_fallback_comment (app,
-								     kf,
-								     keys[i],
-								     error))
-				return FALSE;
-		}
-	}
-
-	/* all applications require icons */
-	if (as_app_get_icons(app)->len == 0)
-		as_app_add_veto (app, "%s has no icon", app_id);
-
-	return TRUE;
+	return as_app_parse_desktop_kf (app, kf, flags, error);
 }
