@@ -2,26 +2,11 @@
  *
  * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 #include "config.h"
 
-#include <fnmatch.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libsoup/soup.h>
 #include <libsoup/soup-status.h>
@@ -161,10 +146,16 @@ as_app_validate_has_first_word_capital (AsAppValidateHelper *helper, const gchar
 static void
 as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 {
-	gboolean require_sentence_case = TRUE;
+	gboolean require_sentence_case = FALSE;
 	guint str_len;
-	guint length_li_max = 100;
-	guint length_li_min = 20;
+	guint length_li_max = 500;
+	guint length_li_min = 3;
+
+	/* make the requirements more strict */
+	if ((helper->flags & AS_APP_VALIDATE_FLAG_STRICT) > 0) {
+		require_sentence_case = TRUE;
+		length_li_max = 250;
+	}
 
 	/* relax the requirements a bit */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
@@ -194,7 +185,7 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 				     "<li> is too long [%s] maximum is %u chars",
 				     text, length_li_max);
 	}
-	if (ai_app_validate_fullstop_ending (text)) {
+	if (require_sentence_case && ai_app_validate_fullstop_ending (text)) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
 				     "<li> cannot end in '.' [%s]", text);
@@ -216,9 +207,9 @@ as_app_validate_description_li (const gchar *text, AsAppValidateHelper *helper)
 static void
 as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper)
 {
-	gboolean require_sentence_case = TRUE;
-	guint length_para_max = 600;
-	guint length_para_min = 50;
+	gboolean require_sentence_case = FALSE;
+	guint length_para_max = 1000;
+	guint length_para_min = 10;
 	guint str_len;
 
 	/* empty */
@@ -229,11 +220,15 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 		return;
 	}
 
+	/* make the requirements more strict */
+	if ((helper->flags & AS_APP_VALIDATE_FLAG_STRICT) > 0) {
+		require_sentence_case = TRUE;
+	}
+
 	/* relax the requirements a bit */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
-		length_para_max = 1000;
-		length_para_min = 10;
-		require_sentence_case = FALSE;
+		length_para_max = 2000;
+		length_para_min = 5;
 	}
 
 	/* previous was short */
@@ -275,7 +270,8 @@ as_app_validate_description_para (const gchar *text, AsAppValidateHelper *helper
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
 				     "<p> requires sentence case [%s]", text);
 	}
-	if (text[str_len - 1] != '.' &&
+	if (require_sentence_case &&
+	    text[str_len - 1] != '.' &&
 	    text[str_len - 1] != '!' &&
 	    text[str_len - 1] != ':') {
 		ai_app_validate_add (helper,
@@ -291,7 +287,7 @@ as_app_validate_description_list (const gchar *text,
 				  gboolean allow_short_para,
 				  AsAppValidateHelper *helper)
 {
-	guint length_para_before_list = 300;
+	guint length_para_before_list = 20;
 
 	/* relax the requirements a bit */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
@@ -524,8 +520,8 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	    as_image_get_width (im) != screenshot_width) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> width did not match specified [%s]",
-				     url);
+				     "<screenshot> width (%u) did not match specified (%u) [%s]",
+				     as_image_get_width (im), screenshot_width, url);
 	}
 
 	/* check height matches */
@@ -533,16 +529,16 @@ ai_app_validate_image_check (AsImage *im, AsAppValidateHelper *helper)
 	    as_image_get_height (im) != screenshot_height) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> height did not match specified [%s]",
-				     url);
+				     "<screenshot> height (%u) did not match specified (%u) [%s]",
+				     as_image_get_height (im), screenshot_height, url);
 	}
 
 	/* check size is reasonable */
 	if (screenshot_width < ss_size_width_min) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
-				     "<screenshot> width too small [%s] minimum is %upx",
-				     url, ss_size_width_min);
+				     "<screenshot> width (%u) too small [%s] minimum is %upx",
+				     screenshot_width, url, ss_size_width_min);
 	}
 	if (screenshot_height < ss_size_height_min) {
 		ai_app_validate_add (helper,
@@ -703,8 +699,17 @@ as_app_validate_icons (AsApp *app, AsAppValidateHelper *helper)
 
 	/* just check the default icon */
 	icon = as_app_get_icon_default (app);
-	if (icon == NULL)
+	if (icon == NULL) {
+		AsFormat *fmt = as_app_get_format_default (app);
+		if (fmt != NULL &&
+		    as_format_get_kind (fmt) == AS_FORMAT_KIND_APPSTREAM &&
+		    as_app_get_kind (app) == AS_APP_KIND_DESKTOP) {
+			ai_app_validate_add (helper,
+					     AS_PROBLEM_KIND_TAG_MISSING,
+					     "desktop application has no icon");
+		}
 		return;
+	}
 
 	/* check the content is correct */
 	icon_kind = as_icon_get_kind (icon);
@@ -760,7 +765,7 @@ as_app_validate_screenshots (AsApp *app, AsAppValidateHelper *helper)
 	AsScreenshot *ss;
 	GPtrArray *screenshots;
 	gboolean screenshot_has_default = FALSE;
-	guint number_screenshots_max = 5;
+	guint number_screenshots_max = 25;
 	guint number_screenshots_min = 1;
 	guint i;
 
@@ -773,6 +778,8 @@ as_app_validate_screenshots (AsApp *app, AsAppValidateHelper *helper)
 	/* firmware does not need screenshots */
 	if (as_app_get_kind (app) == AS_APP_KIND_FIRMWARE ||
 	    as_app_get_kind (app) == AS_APP_KIND_DRIVER ||
+	    as_app_get_kind (app) == AS_APP_KIND_RUNTIME ||
+	    as_app_get_kind (app) == AS_APP_KIND_ADDON ||
 	    as_app_get_kind (app) == AS_APP_KIND_LOCALIZATION)
 		number_screenshots_min = 0;
 
@@ -826,16 +833,22 @@ as_app_validate_release (AsApp *app,
 {
 	const gchar *tmp;
 	guint64 timestamp;
-	guint number_para_max = 3;
+	guint number_para_max = 10;
 	guint number_para_min = 1;
 	gboolean required_timestamp = TRUE;
+	const guint64 MAX_TZ_OFFSET = 14 * 60 * 60; /* UTC+14 is the biggest offset */
 
 	/* relax the requirements a bit */
 	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
-		number_para_max = 4;
+		number_para_max = 20;
 		required_timestamp = FALSE;
 	}
 
+	/* make the requirements more strict */
+	if ((helper->flags & AS_APP_VALIDATE_FLAG_STRICT) > 0) {
+		number_para_max = 4;
+	}
+
 	/* check version */
 	tmp = as_release_get_version (release);
 	if (tmp == NULL) {
@@ -858,7 +871,7 @@ as_app_validate_release (AsApp *app,
 	}
 
 	/* check the timestamp is not in the future */
-	if (timestamp > (guint64) g_get_real_time () / G_USEC_PER_SEC) {
+	if (timestamp > (guint64) g_get_real_time () / G_USEC_PER_SEC + MAX_TZ_OFFSET) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_ATTRIBUTE_INVALID,
 				     "<release> timestamp is in the future");
@@ -921,6 +934,7 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 {
 	GPtrArray *releases;
 	AsFormat *format;
+	gboolean require_release = FALSE;
 
 	/* only for AppData */
 	format = as_app_get_format_default (app);
@@ -928,7 +942,23 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 	    as_format_get_kind (format) != AS_FORMAT_KIND_METAINFO)
 		return TRUE;
 
+	/* make the requirements more strict */
+	if ((helper->flags & AS_APP_VALIDATE_FLAG_RELAX) == 0) {
+		/* only for desktop and console apps */
+		if (as_app_get_kind (app) == AS_APP_KIND_DESKTOP ||
+		    as_app_get_kind (app) == AS_APP_KIND_CONSOLE) {
+			require_release = TRUE;
+		}
+	}
+
+	/* require releases */
 	releases = as_app_get_releases (app);
+	if (require_release && releases->len == 0) {
+		ai_app_validate_add (helper,
+				     AS_PROBLEM_KIND_TAG_MISSING,
+				     "<release> required");
+		return TRUE;
+	}
 	for (guint i = 0; i < releases->len; i++) {
 		AsRelease *release = g_ptr_array_index (releases, i);
 		if (!as_app_validate_release (app, release, helper, error))
@@ -944,7 +974,7 @@ as_app_validate_releases (AsApp *app, AsAppValidateHelper *helper, GError **erro
 			const gchar *version_old = as_release_get_version (release_old);
 			if (version == NULL || version_old == NULL)
 				continue;
-			if (as_utils_vercmp (version, version_old) > 0) {
+			if (as_utils_vercmp_full (version, version_old, AS_VERSION_COMPARE_FLAG_NONE) > 0) {
 				ai_app_validate_add (helper,
 						     AS_PROBLEM_KIND_TAG_INVALID,
 						     "<release> versions are not in order "
@@ -1016,6 +1046,12 @@ as_app_validate_license (const gchar *license_text, GError **error)
 static gboolean
 as_app_validate_is_content_license_id (const gchar *license_id)
 {
+	if (g_strcmp0 (license_id, "@FSFAP") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@MIT") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@0BSD") == 0)
+		return TRUE;
 	if (g_strcmp0 (license_id, "@CC0-1.0") == 0)
 		return TRUE;
 	if (g_strcmp0 (license_id, "@CC-BY-3.0") == 0)
@@ -1032,7 +1068,11 @@ as_app_validate_is_content_license_id (const gchar *license_id)
 		return TRUE;
 	if (g_strcmp0 (license_id, "@GFDL-1.3") == 0)
 		return TRUE;
-	if (g_strcmp0 (license_id, "@FSFAP") == 0)
+	if (g_strcmp0 (license_id, "@BSL-1.0") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@FTL") == 0)
+		return TRUE;
+	if (g_strcmp0 (license_id, "@FSFUL") == 0)
 		return TRUE;
 	return FALSE;
 }
@@ -1102,6 +1142,11 @@ static gboolean
 as_app_validate_check_id_char (const gchar c)
 {
 	const gchar valid[] = { '-', '_', '.', '\0' };
+	const gchar invalid[] = { '/', '\\', '\0' };
+	for (guint i = 0; invalid[i] != '\0'; i++) {
+		if (invalid[i] == c)
+			return FALSE;
+	}
 	for (guint i = 0; valid[i] != '\0'; i++) {
 		if (valid[i] == c)
 			return TRUE;
@@ -1138,7 +1183,7 @@ as_app_validate_check_id (AsAppValidateHelper *helper, const gchar *id)
  * @flags: the #AsAppValidateFlags to use, e.g. %AS_APP_VALIDATE_FLAG_NONE
  * @error: A #GError or %NULL.
  *
- * Validates data in the instance for style and consitency.
+ * Validates data in the instance for style and consistency.
  *
  * Returns: (transfer container) (element-type AsProblem): A list of problems, or %NULL
  *
@@ -1161,24 +1206,26 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 	const gchar *update_contact;
 	gboolean deprecated_failure = FALSE;
 	gboolean require_appstream_spec_only = FALSE;
-	gboolean require_contactdetails = TRUE;
+	gboolean require_contactdetails = FALSE;
 	gboolean require_copyright = FALSE;
+	gboolean require_description = FALSE;
 	gboolean require_project_license = FALSE;
-	gboolean require_sentence_case = TRUE;
+	gboolean require_sentence_case = FALSE;
 	gboolean require_translations = FALSE;
 	gboolean require_url = TRUE;
 	gboolean require_content_license = TRUE;
 	gboolean require_name = TRUE;
-	gboolean require_translation = TRUE;
-	gboolean require_content_rating = TRUE;
+	gboolean require_translation = FALSE;
+	gboolean require_content_rating = FALSE;
+	gboolean require_name_shorter_than_summary = FALSE;
 	gboolean validate_license = TRUE;
 	gboolean ret;
-	guint length_name_max = 30;
+	guint length_name_max = 60;
 	guint length_name_min = 3;
-	guint length_summary_max = 100;
+	guint length_summary_max = 200;
 	guint length_summary_min = 8;
-	guint number_para_max = 4;
-	guint number_para_min = 2;
+	guint number_para_max = 10;
+	guint number_para_min = 1;
 	guint str_len;
 	g_autoptr(GList) keys = NULL;
 	g_autoptr(AsAppValidateHelper) helper = g_new0 (AsAppValidateHelper, 1);
@@ -1193,18 +1240,23 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 		return NULL;
 	}
 
+	/* only for desktop and console apps */
+	if (as_app_get_kind (app) == AS_APP_KIND_DESKTOP ||
+	    as_app_get_kind (app) == AS_APP_KIND_CONSOLE) {
+		require_content_rating = TRUE;
+		require_description = TRUE;
+	}
+
 	/* relax the requirements a bit */
 	if ((flags & AS_APP_VALIDATE_FLAG_RELAX) > 0) {
 		length_name_max = 100;
 		length_summary_max = 200;
-		require_contactdetails = FALSE;
 		require_content_license = FALSE;
 		validate_license = FALSE;
 		require_url = FALSE;
-		number_para_max = 10;
+		number_para_max = 20;
 		number_para_min = 1;
 		require_sentence_case = FALSE;
-		require_translation = FALSE;
 		require_content_rating = FALSE;
 		switch (as_format_get_kind (format)) {
 		case AS_FORMAT_KIND_METAINFO:
@@ -1224,6 +1276,12 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 		require_project_license = TRUE;
 		require_content_license = TRUE;
 		require_appstream_spec_only = TRUE;
+		require_sentence_case = TRUE;
+		require_name_shorter_than_summary = TRUE;
+		require_contactdetails = TRUE;
+		require_translation = TRUE;
+		number_para_min = 2;
+		number_para_max = 4;
 	}
 
 	/* addons don't need such a long description */
@@ -1313,6 +1371,43 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 		}
 	}
 
+	/* categories */
+	if (as_format_get_kind (format) == AS_FORMAT_KIND_APPSTREAM &&
+	    as_app_get_kind (app) == AS_APP_KIND_DESKTOP) {
+		GPtrArray *categories = as_app_get_categories (app);
+		guint nr_toplevel_cats = 0;
+		const gchar *cats[] = { "AudioVideo",
+					"Development",
+					"Education",
+					"Game",
+					"Graphics",
+					"Network",
+					"Office",
+					"Science",
+					"Settings",
+					"System",
+					"Utility",
+					NULL };
+		for (guint i = 0; i < categories->len; i++) {
+			const gchar *cat = g_ptr_array_index (categories, i);
+			for (guint j = 0; cats[j] != NULL; j++) {
+				if (g_strcmp0 (cats[j], cat) == 0)
+					nr_toplevel_cats++;
+			}
+		}
+		if (nr_toplevel_cats == 0) {
+			ai_app_validate_add (helper,
+					     AS_PROBLEM_KIND_TAG_MISSING,
+					     "<category> must include main categories "
+					     "from the desktop entry spec");
+		} else if (nr_toplevel_cats > 3) {
+			ai_app_validate_add (helper,
+					     AS_PROBLEM_KIND_TAG_MISSING,
+					     "too many main <category> types: %u",
+					     nr_toplevel_cats);
+		}
+	}
+
 	/* translation */
 	if (require_translation &&
 	    as_format_get_kind (format) == AS_FORMAT_KIND_APPDATA &&
@@ -1455,12 +1550,12 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 	}
 
 	/* games require a content rating */
-	if (require_content_rating && as_app_has_category (app, "Game")) {
+	if (require_content_rating) {
 		GPtrArray *ratings = as_app_get_content_ratings (app);
 		if (ratings->len == 0) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_TAG_MISSING,
-					     "<content_rating> required for game "
+					     "<content_rating> required "
 					     "[use https://odrs.gnome.org/oars]");
 		}
 	}
@@ -1558,7 +1653,8 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 					     "<summary> is too long [%s] maximum is %u chars",
 					     summary, length_summary_max);
 		}
-		if (ai_app_validate_fullstop_ending (summary)) {
+		if (require_sentence_case &&
+		    ai_app_validate_fullstop_ending (summary)) {
 			ai_app_validate_add (helper,
 					     AS_PROBLEM_KIND_STYLE_INCORRECT,
 					     "<summary> cannot end in '.' [%s]",
@@ -1582,14 +1678,21 @@ as_app_validate (AsApp *app, guint32 flags, GError **error)
 				     AS_PROBLEM_KIND_TAG_MISSING,
 				     "<summary> is not present");
 	}
-	if (summary != NULL && name != NULL &&
+	if (require_name_shorter_than_summary &&
+	    summary != NULL && name != NULL &&
 	    strlen (summary) < strlen (name)) {
 		ai_app_validate_add (helper,
 				     AS_PROBLEM_KIND_STYLE_INCORRECT,
 				     "<summary> is shorter than <name>");
 	}
 	description = as_app_get_description (app, "C");
-	if (description != NULL) {
+	if (description == NULL) {
+		if (require_description) {
+			ai_app_validate_add (helper,
+					     AS_PROBLEM_KIND_TAG_MISSING,
+					     "<description> required");
+		}
+	} else {
 		ret = as_app_validate_description (description,
 						   helper,
 						   number_para_min,
