@@ -2,21 +2,7 @@
  *
  * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 /**
@@ -506,7 +492,7 @@ as_node_to_xml_string (GString *xml,
  * @text_len: length of @text
  *
  * Converts pretty-formatted source text into a format suitable for AppStream.
- * This might include joining paragraphs, supressing newlines or doing other
+ * This might include joining paragraphs, suppressing newlines or doing other
  * sanity checks to the text.
  *
  * Returns: (transfer full): a new string
@@ -949,6 +935,7 @@ as_node_from_file (GFile *file,
 	gsize chunk_size = 32 * 1024;
 	gssize len;
 	g_autofree gchar *data = NULL;
+	g_autofree gchar *mime_type = NULL;
 	g_autoptr(GMarkupParseContext) ctx = NULL;
 	g_autoptr(GConverter) conv = NULL;
 	g_autoptr(GFileInfo) info = NULL;
@@ -975,18 +962,22 @@ as_node_from_file (GFile *file,
 	if (file_stream == NULL)
 		return NULL;
 	content_type = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE);
-	if (g_strcmp0 (content_type, "application/gzip") == 0 ||
-	    g_strcmp0 (content_type, "application/x-gzip") == 0) {
-		conv = G_CONVERTER (g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP));
-		stream_data = g_converter_input_stream_new (file_stream, conv);
-	} else if (g_strcmp0 (content_type, "application/xml") == 0) {
-		stream_data = g_object_ref (file_stream);
-	} else {
+	mime_type = g_content_type_get_mime_type (content_type);
+	if (mime_type != NULL) {
+		if (g_strcmp0 (mime_type, "application/gzip") == 0 ||
+			g_strcmp0 (mime_type, "application/x-gzip") == 0) {
+			conv = G_CONVERTER (g_zlib_decompressor_new (G_ZLIB_COMPRESSOR_FORMAT_GZIP));
+			stream_data = g_converter_input_stream_new (file_stream, conv);
+		} else if (g_strcmp0 (mime_type, "application/xml") == 0 ||
+				   g_strcmp0 (mime_type, "text/xml") == 0) {
+			stream_data = g_object_ref (file_stream);
+		}
+	}
+	if (stream_data == NULL) {
 		g_set_error (error,
 			     AS_NODE_ERROR,
 			     AS_NODE_ERROR_FAILED,
-			     "cannot process file of type %s",
-			     content_type);
+			     "cannot process file of type %s", content_type);
 		return NULL;
 	}
 
