@@ -2,21 +2,7 @@
  *
  * Copyright (C) 2014 Richard Hughes <richard@hughsie.com>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 /**
@@ -129,6 +115,7 @@ AsProblemKind
 as_problem_get_kind (AsProblem *problem)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_val_if_fail (AS_IS_PROBLEM (problem), AS_PROBLEM_KIND_UNKNOWN);
 	return priv->kind;
 }
 
@@ -146,6 +133,7 @@ guint
 as_problem_get_line_number (AsProblem *problem)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_val_if_fail (AS_IS_PROBLEM (problem), 0);
 	return priv->line_number;
 }
 
@@ -163,6 +151,7 @@ const gchar *
 as_problem_get_message (AsProblem *problem)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_val_if_fail (AS_IS_PROBLEM (problem), NULL);
 	return priv->message;
 }
 
@@ -179,6 +168,7 @@ void
 as_problem_set_kind (AsProblem *problem, AsProblemKind kind)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_if_fail (AS_IS_PROBLEM (problem));
 	priv->kind = kind;
 }
 
@@ -195,6 +185,7 @@ void
 as_problem_set_line_number (AsProblem *problem, guint line_number)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_if_fail (AS_IS_PROBLEM (problem));
 	priv->line_number = line_number;
 }
 
@@ -211,6 +202,7 @@ void
 as_problem_set_message (AsProblem *problem, const gchar *message)
 {
 	AsProblemPrivate *priv = GET_PRIVATE (problem);
+	g_return_if_fail (AS_IS_PROBLEM (problem));
 	g_free (priv->message);
 	priv->message = g_strdup (message);
 }
