@@ -3,21 +3,7 @@
  * Copyright (C) 2014-2016 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2011 Paolo Bacchilega <paobac@src.gnome.org>
  *
- * Licensed under the GNU Lesser General Public License Version 2.1
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
+ * SPDX-License-Identifier: LGPL-2.1+
  */
 
 /**
@@ -32,13 +18,18 @@
 
 #include "config.h"
 
-#include <fnmatch.h>
 #include <string.h>
 #include <archive_entry.h>
 #include <archive.h>
 #include <libsoup/soup.h>
 #include <stdlib.h>
+#ifndef _WIN32
 #include <uuid.h>
+#endif
+
+#ifdef HAVE_RPM
+#include <rpm/rpmlib.h>
+#endif
 
 #include "as-app-private.h"
 #include "as-enums.h"
@@ -622,6 +613,7 @@ as_utils_license_to_spdx (const gchar *license)
 		{ "Python",			"Python-2.0" },
 		{ "QPL",			"QPL-1.0" },
 		{ "SPL",			"SPL-1.0" },
+		{ "UPL",			"UPL-1.0" },
 		{ "zlib",			"Zlib" },
 		{ "ZPLv2.0",			"ZPL-2.0" },
 		{ "Unlicense",			"CC0-1.0" },
@@ -1190,8 +1182,7 @@ as_utils_install_xml (const gchar *filename,
 	/* actually copy file */
 	file_dest = g_file_new_for_path (path_dest);
 	if (!g_file_copy (file_src, file_dest,
-			  G_FILE_COPY_OVERWRITE |
-			  G_FILE_COPY_TARGET_DEFAULT_PERMS,
+			  G_FILE_COPY_OVERWRITE,
 			  NULL, NULL, NULL, error))
 		return FALSE;
 
@@ -1387,39 +1378,17 @@ as_utils_vercmp_chunk (const gchar *str1, const gchar *str2)
 	return as_utils_vercmp_char (str1[i], str2[i]);
 }
 
-/**
- * as_utils_vercmp:
- * @version_a: the release version, e.g. 1.2.3
- * @version_b: the release version, e.g. 1.2.3.1
- *
- * Compares version numbers for sorting.
- *
- * Returns: -1 if a < b, +1 if a > b, 0 if they are equal, and %G_MAXINT on error
- *
- * Since: 0.3.5
- */
-gint
-as_utils_vercmp (const gchar *version_a, const gchar *version_b)
+static gint
+as_utils_vercmp_internal (const gchar *version_a,
+                          const gchar *version_b)
 {
 	guint longest_split;
-	g_autofree gchar *str_a = NULL;
-	g_autofree gchar *str_b = NULL;
 	g_auto(GStrv) split_a = NULL;
 	g_auto(GStrv) split_b = NULL;
 
-	/* sanity check */
-	if (version_a == NULL || version_b == NULL)
-		return G_MAXINT;
+	split_a = g_strsplit (version_a, ".", -1);
+	split_b = g_strsplit (version_b, ".", -1);
 
-	/* optimisation */
-	if (g_strcmp0 (version_a, version_b) == 0)
-		return 0;
-
-	/* split into sections, and try to parse */
-	str_a = as_utils_version_parse (version_a);
-	str_b = as_utils_version_parse (version_b);
-	split_a = g_strsplit (str_a, ".", -1);
-	split_b = g_strsplit (str_b, ".", -1);
 	longest_split = MAX (g_strv_length (split_a), g_strv_length (split_b));
 	for (guint i = 0; i < longest_split; i++) {
 		gchar *endptr_a = NULL;
@@ -1456,6 +1425,63 @@ as_utils_vercmp (const gchar *version_a, const gchar *version_b)
 	return 0;
 }
 
+/**
+ * as_utils_vercmp_full:
+ * @version_a: the release version, e.g. 1.2.3
+ * @version_b: the release version, e.g. 1.2.3.1
+ * @flags: some #AsVersionCompareFlag
+ *
+ * Compares version numbers for sorting.
+ *
+ * Returns: -1 if a < b, +1 if a > b, 0 if they are equal, and %G_MAXINT on error
+ *
+ * Since: 0.7.15
+ */
+gint
+as_utils_vercmp_full (const gchar *version_a,
+		      const gchar *version_b,
+		      AsVersionCompareFlag flags)
+{
+	/* sanity check */
+	if (version_a == NULL || version_b == NULL)
+		return G_MAXINT;
+
+	/* optimisation */
+	if (g_strcmp0 (version_a, version_b) == 0)
+		return 0;
+
+	if (flags & AS_VERSION_COMPARE_FLAG_USE_HEURISTICS) {
+		/* split into sections, and try to parse */
+		g_autofree gchar *str_a = as_utils_version_parse (version_a);
+		g_autofree gchar *str_b = as_utils_version_parse (version_b);
+		return as_utils_vercmp_internal (str_a, str_b);
+	} else {
+#ifdef HAVE_RPM
+		return rpmvercmp (version_a, version_b);
+#else
+		return as_utils_vercmp_internal (version_a, version_b);
+#endif
+	}
+}
+
+/**
+ * as_utils_vercmp:
+ * @version_a: the release version, e.g. 1.2.3
+ * @version_b: the release version, e.g. 1.2.3.1
+ *
+ * Compares version numbers for sorting.
+ *
+ * Returns: -1 if a < b, +1 if a > b, 0 if they are equal, and %G_MAXINT on error
+ *
+ * Since: 0.3.5
+ */
+gint
+as_utils_vercmp (const gchar *version_a, const gchar *version_b)
+{
+	return as_utils_vercmp_full (version_a, version_b,
+				     AS_VERSION_COMPARE_FLAG_USE_HEURISTICS);
+}
+
 /**
  * as_ptr_array_find_string:
  * @array: gchar* array
@@ -1501,6 +1527,13 @@ as_utils_guid_from_data (const gchar *namespace_id,
 			 gsize data_len,
 			 GError **error)
 {
+#ifdef _WIN32
+	g_set_error_literal (error,
+			     AS_UTILS_ERROR,
+			     AS_UTILS_ERROR_FAILED,
+			     "not supported");
+	return FALSE;
+#else
 	gchar guid_new[37]; /* 36 plus NUL */
 	gsize digestlen = 20;
 	guint8 hash[20];
@@ -1540,6 +1573,7 @@ as_utils_guid_from_data (const gchar *namespace_id,
 	/* return as a string */
 	uuid_unparse (uu_new, guid_new);
 	return g_strdup (guid_new);
+#endif
 }
 
 /**
@@ -1555,12 +1589,20 @@ as_utils_guid_from_data (const gchar *namespace_id,
 gboolean
 as_utils_guid_is_valid (const gchar *guid)
 {
+#ifdef _WIN32
+	/* XXX Ideally we should set a GError but this was already a public
+	 * API, and it doesn't have such parameter.
+	 */
+	g_printerr ("%s: not supported\n", G_STRFUNC);
+	return FALSE;
+#else
 	gint rc;
 	uuid_t uu;
 	if (guid == NULL)
 		return FALSE;
 	rc = uuid_parse (guid, uu);
 	return rc == 0;
+#endif
 }
 
 /**
@@ -2068,7 +2110,7 @@ as_utils_appstream_id_build (const gchar *str)
  * @str: a string
  *
  * Checks to see if a string is a valid AppStream ID. A valid AppStream ID only
- * contains alpha-numeric chars, dots and dashes.
+ * contains alphanumeric chars, dots and dashes.
  *
  * Returns: %TRUE if the string is a valid AppStream ID
  *
